import json
import urllib.parse
import boto3
import pprint

print('Loading function')

s3_client = boto3.client('s3')
s3_resource = boto3.resource('s3') 


def lambda_handler(event, context):
    # Get the object from the event and show its content type
    bucket = event['Records'][0]['s3']['bucket']['name']
    key = urllib.parse.unquote_plus(event['Records'][0]['s3']['object']['key'], encoding='utf-8')
    complete_path_as_array = key.split('/', 1)
    origin_folder_name = complete_path_as_array[0]
    qualified_path = ''
    if len(complete_path_as_array) > 1 :
        qualified_path = complete_path_as_array[1]
    image_content_types = ["image/png", "image/jpeg"]
    document_content_types = ["application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                              "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                              "text/csv", "text/plain"]
    try:
        uploaded_object = s3_client.get_object(Bucket=bucket,Key=key)
        size = event['Records'][0]['s3']['object']['size']
        content_type = uploaded_object['ContentType']
        pprint.pprint("Content Type of uploaded object is " + uploaded_object['ContentType'])
        if size == 0 and content_type == "application/x-directory; charset=UTF-8":
            pprint.pprint("it is a folder")
        elif content_type in image_content_types:
            if origin_folder_name != "pictures":
                copy_object_from_one_folder_to_another(bucket, qualified_path, origin_folder_name, 'pictures')
                delete_object_from_bucket(bucket, key)
            else:
                pprint.pprint("Already in correct folder pictures")
        elif content_type in document_content_types:
            if origin_folder_name != "documents":
                copy_object_from_one_folder_to_another(bucket, qualified_path, origin_folder_name, 'documents')
                delete_object_from_bucket(bucket, key)
            else:
                pprint.pprint("Already in correct folder documents")
        else:
            if origin_folder_name != "miscellaneous":
                copy_object_from_one_folder_to_another(bucket, qualified_path, origin_folder_name, 'miscellaneous')
                delete_object_from_bucket(bucket, key)
            else:
                pprint.pprint("Already in correct folder miscellaneous")
           
        return 'success'
    except Exception as e:
        print(e)
        print('Error getting object {} from bucket {}. Make sure they exist and your bucket is in the same region as this function.'.format(key, bucket))
        raise e

def delete_object_from_bucket(bucket_name, file_name):
    response = s3_client.delete_object(Bucket=bucket_name, Key=file_name)
    return response

def copy_object_from_one_folder_to_another(bucket_name, file_name, source_folder, destination_folder):
    # Declaring the source to be copied
    key = ''
    if file_name == '':
        key = source_folder
    else :
        key = f"{source_folder}/{file_name}"
    copy_source = {"Bucket": bucket_name, "Key": key}
    bucket = s3_resource.Bucket(bucket_name)
    # Copying the files to another folder
    origin_file_name = ''
    if file_name == '':
        origin_file_name = source_folder
    else :
        origin_file_name = file_name
    response = bucket.copy(copy_source, f"{destination_folder}/{origin_file_name}")
    pprint.pprint(response)
    return response
    